/*
 * File   :     chext.java
 * Author :     Tis Veugen
 * Date   :     2006-08-11
 * Version:     1.0
 */

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class chext extends Frame
   implements ActionListener, TextListener
{
  private Button                mInput;
  private Button                mChange;

  private TextField             mInputTextField;
  private TextField             mExtensionTextField;

  private String                mDirectory = "";
  private String                msPattern;
  private String                msExtension;

  public                        chext()
  {
    setTitle( "chext (v1.0)" );

    GridBagLayout               lGbl = new GridBagLayout();
    setLayout( lGbl );

    mInput  = addButton( "Input" );
    mChange = addButton( "ChangeTo" );

    mInputTextField = addTextField( 40 );
    mExtensionTextField = addTextField( 40 );

    GridBagConstraints          lGbc = new GridBagConstraints();

    lGbc.anchor = GridBagConstraints.CENTER;
    lGbc.fill   = GridBagConstraints.HORIZONTAL;
    lGbc.weightx = 20;
    lGbc.weighty = 10;

    add( mInput,  lGbc, 0, 0, 1, 1 );
    add( mChange, lGbc, 0, 1, 1, 1 );

    lGbc.fill = GridBagConstraints.NONE;
    add( mInputTextField,     lGbc, 1, 0, 3, 1 );
    add( mExtensionTextField, lGbc, 1, 1, 3, 1 );

    addWindowListener( new WindowAdapter()
                       {
                         public void windowClosing(
                         WindowEvent evt )
                         {
                           setVisible( false );
                           System.exit( 0 );
                         }
                       }
                     );

    setLocation( 100, 100 );
    setSize( 400, 100 );
}

  public void                   add(
  Component                     oComponent,
  GridBagConstraints            oGbc,
  int                           x,
  int                           y,
  int                           w,
  int                           h )
  {
    oGbc.gridx = x;
    oGbc.gridy = y;
    oGbc.gridwidth  = w;
    oGbc.gridheight = h;
    add( oComponent, oGbc );
  }

  public Button                 addButton(
  String                        oName )
  {
    Button                      lButton = new Button( oName );

    lButton.addActionListener( this );

    return lButton;
  }

  public TextField              addTextField(
  int                           size )
  {
    TextField                   lTextField = new TextField( "", size );

    lTextField.addTextListener( this );

    return lTextField;
  }

  public void                   ActionInput()
  {
    String                      lFilename;

    lFilename = SetFile( "*.*", "Input file", FileDialog.LOAD );
    if ( lFilename != null )
    {
      mInputTextField.setText( lFilename );
      int ix = lFilename.lastIndexOf( '.' );
      if ( ix >= 0 )
      {
		    String                  lExtension;
		    lExtension = lFilename.substring( ix+1 );

	      mExtensionTextField.setText( lExtension );
	    }
    }
  }

  private String                SetFile(
  String                        oFilter,
  String                        oHeader,
  int                           iMode )
  {
    FileDialog                  lFd;
    String                      lFilename;

    lFd = new FileDialog( this      // mFrame
                        , oHeader
                        , iMode );
    lFd.setDirectory( mDirectory );
    lFd.setFile( oFilter );
    lFd.show();

    lFilename = lFd.getFile();
    if ( lFilename != null )
    {
      mDirectory = lFd.getDirectory();
      lFilename = mDirectory + lFilename;
    }

    return lFilename;
  }

  private void                  PrintDialog(
  boolean                       bDialog,
  Frame                         oFrame,
  String                        oHeader,
  String                        oText )
  {
    if ( bDialog )
    {
      new PbnDialog( oFrame, oHeader, oText );
    }
    else
    {
      System.out.println( oText );
    }
  }

  private boolean               MyRename(
  String                        oSourceFilename,
  String                        oTargetFilename )
  {
    File                        lSourceFile = new File( oSourceFilename );
    File                        lTargetFile = new File( oTargetFilename );

    try
    {
      lTargetFile.delete();
      lSourceFile.renameTo( lTargetFile );
    }
    catch ( Exception e )
    {
      return false;
    }
    return true;
  }

  private boolean               ActionChext(
  String                        oInputFilename,
  String                        oExtension,
  boolean                       bDialog )
  {
    String                      lString;
    boolean                     bOK = true;

    File                        lFile = new File( oInputFilename );
    msPattern = lFile.getName();

    {
      if ( bDialog )
      {
        this.setCursor( new Cursor( Cursor.WAIT_CURSOR ) );
      }

      msExtension = oExtension;
      HandleDir( lFile.getParent(), bDialog );

      if ( bDialog )
      {
        this.setCursor( new Cursor( Cursor.DEFAULT_CURSOR ) );
      }
    }

    return bOK;
  }

  private void                  HandleDir(
  String                        oDirname,
  boolean                       bDialog )
  {
    File                        lDirectory;
    String []                   lFiles;

    try
    {
      lDirectory = new File( oDirname );
      lFiles = lDirectory.list();
      if ( lFiles == null )
      { /*
         * No files in this dir.
         */
        return;
      }
    }
    catch ( Exception e )
    {
      System.err.println( "Can't construct directory" );
      return;
    }

    for ( int i = 0; i < lFiles.length; i++ )
    {
      File lFile = new File( oDirname, lFiles[ i ] );
      if ( lFile.isFile() )
      {
        if ( PatternMatch.Match( lFile.getName(), msPattern ) )
        {
          HandleFile( lFile, bDialog );
        }
      }
    }

    for ( int i = 0; i < lFiles.length; i++ )
    {
      File lFile = new File( oDirname, lFiles[ i ] );
      if ( ! lFile.isFile() )
      {
        HandleDir( lFile.getPath(), bDialog );
      }
    }
  }

  private void                  HandleFile(
  File                          lFile,
  boolean                       bDialog )
  {
    boolean                     bOk = true;

    String                    	sName = lFile.toString();
    int ix = sName.lastIndexOf( '.' );
    if ( ix >= 0 )
    {
		  String                  	sNewName;
		  sNewName = sName.substring( 0, ix ) + "." + msExtension;

			bOk = MyRename( sName, sNewName );
			if ( ! bOk )
			{
		    PrintDialog( true, this, "Error", sName );
		  }
		}
  }

  private void                  DoAction()
  {
    ActionChext( mInputTextField.getText()
               , mExtensionTextField.getText()
               , true );
    PrintDialog( true, this, "Print Result", "Ready" );
  }

  public void                   actionPerformed(
  ActionEvent                   evt )
  {
    String                      lName = evt.getActionCommand();
    String                      lFilename;

    if ( lName.equals( "Input" ) )
    {
      ActionInput();
    }
    else
    if ( lName.equals( "ChangeTo" ) )
    {
      DoAction();
    }
  }

  public void                   textValueChanged(
  TextEvent                     evt )
  {
  }

  public static void            main(
  String []                     args )
  {
    Frame                       lchext;

    lchext = new chext();
    lchext.show();
  }
}

class PbnDialog extends Dialog
{
  public                        PbnDialog(
  Frame                         oParent,
  String                        oHeader,
  String                        oText )
  {
    super( oParent, oHeader, true );

    Panel                       lPanel = new Panel();
    int                         NrLines;
    int                         Index;

    for ( NrLines = 0; ; NrLines++ )
    {
      Index = oText.indexOf( "\n" );
      if ( Index < 0 )
      {
        lPanel.add( new Label( oText ) );
        break;
      }
      lPanel.add( new Label( oText.substring( 0, Index ) ) );
      oText = oText.substring( Index+1 );
    }
    add( lPanel, "Center" );

    Button                      mOk = new Button( "OK" );
    lPanel = new Panel();
    lPanel.add( mOk );
    add( lPanel, "South" );

    mOk.addActionListener( new ActionListener()
                           {
                             public void actionPerformed(
                             ActionEvent evt )
                             {
                               setVisible( false );
                             }
                           }
                         );

        addWindowListener( new WindowAdapter()
                           {
                             public void windowClosing(
                             WindowEvent evt )
                             {
                               setVisible( false );
                             }
                           }
                         );

    setLocation( 150, 150 );
    setSize( 300, 120 + NrLines * 30 );
    show();
  }
}
